import { reactive, watch, ref, defineEmits, defineExpose } from "vue";
import { ElMessage } from "element-plus";
import { upload, getUploadConfigInfo } from "../axios/model/user";
import COS from "cos-js-sdk-v5";
interface UploadMixin {
  uploadMixinPath: string;
  uploadMixinType: string;
  uploadMixinCredentials: any;
  uploadMixinTimeStamp: number;
}
const uploadMixin: UploadMixin = {
  uploadMixinPath: "",
  uploadMixinType: "",
  uploadMixinCredentials: {},
  uploadMixinTimeStamp: 0,
};
interface CustomAxiosResponse {
  code: number;
  result: {
    bucketName: string;
    regionid: string;
    upLoadPath: string;
    uploadType: string;
    staticDomain: string;
    startTime: string;
    expiredTime: string;
    credentials: {
      tmpSecretId: string;
      tmpSecretKey: string;
      sessionToken: string;
    };
  };
}
export const upLoadFilesHander = (file: any) => {
  return new Promise((resolve) => {
    const loadHandler = ({ success, message }: any) => {
      if (!success) {
        ElMessage({
          message: `上传失败，请重新上传！`,
          type: "warning",
        });
      } else {
        resolve({ success: true, result: message });
      }
    };
    const obj = {
      file: file,
      onSuccess: ({ success, message }: any) => {
        loadHandler({ success, message });
      },
      onError: ({ success, message }: any) => {
        loadHandler({ success: false, message: "" });
      },
    };
    //进行上传
    upLoadHander(obj);
  });
};

export const upLoadHander = async (obj: any) => {
  const { file } = obj;
  const fileName = file.name.split(".");
  const fileInfo = {
    fileTyle: fileName.length > 1 ? fileName[fileName.length - 1] : "",
    fileName: fileName[0],
    biz: obj?.data?.biz || "temp",
    file: file,
  };
  const none = async function hander(noCredentials: boolean) {
    // const res:CustomAxiosResponse = reactive({
    //   code: 0,
    //   result: {
    //     bucketName: "",
    //     regionid: "",
    //     upLoadPath: "",
    //     uploadType: "",
    //     staticDomain: "",
    //     startTime: "",
    //     expiredTime: "",
    //     credentials: {
    //       tmpSecretId: "",
    //       tmpSecretKey: "",
    //       sessionToken: "",
    //     },
    //   },
    // });
    let staticDomain = "";
    let resResult = true;
    //noCredentials变量控制是否走对象存储
    if (noCredentials) {
      const query = {};
      const res:any = await getUploadConfigInfo(query);
      // console.log(res1,'res')
      // console.log(res);
      if (res.code === 200 && res.result) {
        const result = res.result;
        uploadMixin.uploadMixinPath = result.upLoadPath;
        uploadMixin.uploadMixinType = result.uploadType;
        staticDomain = result.staticDomain || "https://cdn.tikcos.cn";
        if (uploadMixin.uploadMixinType === "txcos") {
          const credentials = result.credentials;
          uploadMixin.uploadMixinCredentials = {
            TmpSecretId: credentials?.tmpSecretId,
            TmpSecretKey: credentials?.tmpSecretKey,
            SecurityToken: credentials?.sessionToken,
            StartTime: result.startTime,
            ExpiredTime: result.expiredTime,
          };
        }
      } else {
        resResult = false;
      }
    }
    //resResult判断接口是否正常
    if (resResult) {
      const query = {};
      const res:any = await getUploadConfigInfo(query);
      if (uploadMixin.uploadMixinType == "txcos") {
        //走对象存储
        const cos = new COS({
          getAuthorization: (options: any, callback: any) => {
            callback(uploadMixin.uploadMixinCredentials);
          },
        });
        const tempFolder = fileInfo.biz === "temp" ? "" : `${fileInfo.biz}/`;
        const folder = `${fileInfo.fileName}_${new Date().getTime()}.${
          fileInfo.fileTyle
        }`;
        const result = res.result || {};
        cos.putObject(
          {
            // FilePath: fileInfo.file,
            Bucket: result.bucketName || "tikcos-1257774783",
            Region: result.regionid || "ap-guangzhou",
            Key: uploadMixin.uploadMixinPath + tempFolder + folder,
            Body: fileInfo.file,
            onProgress: function (progressData: any) {
              obj.onProgress && obj.onProgress(progressData);
            },
          },
          async (err: any, data: any) => {
            if (!err && data.statusCode === 200) {
              const imgurl = `${staticDomain}/${uploadMixin.uploadMixinPath}${tempFolder}${folder}`;
              obj.onSuccess({
                success: true,
                message: imgurl,
              });
            } else {
              obj.onError({
                success: false,
                message: data.message || "图片上传失败",
              });
            }
          }
        );
      } else {
        // 其他存储类型暂不支持
        obj.onError({
          success: false,
          message: "服务接口报错",
        });
      }
    } else {
      obj.onError({
        success: false,
        message: "服务接口报错",
      });
    }
  };
  none(true);
};
